package com.topsunit.scanservice.ximai.service;

import com.topsunit.scanservice.ximai.dao.InvmbDao;
import com.topsunit.scanservice.ximai.dao.MoctaDao;
import com.topsunit.scanservice.ximai.dao.MoctbDao;
import com.topsunit.scanservice.ximai.dto.*;
import com.topsunit.scanservice.ximai.dto.mapper.MoctaMapper;
import com.topsunit.scanservice.ximai.entity.Invml;
import com.topsunit.scanservice.ximai.entity.Mocta;
import com.topsunit.scanservice.ximai.entity.Moctb;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * <p>Title: MoctaService</p>
 * <p>Description: 工单单头</p>
 *
 * @author xi.feng
 * @version V1.0
 * @date 2021/11/16
 */
@Service
public class MoctaService {

    @Autowired
    private MoctbDao moctbDao;
    private final MoctaDao moctaDao;

    private final InvmbDao invmbDao;
    private final MoctaMapper moctaMapper;

    public MoctaService(MoctaDao moctaDao, InvmbDao invmbDao, MoctaMapper moctaMapper) {
        this.moctaDao = moctaDao;
        this.invmbDao = invmbDao;
        this.moctaMapper = moctaMapper;
    }

    public Optional<MoctaDocDto> getForDoc(MoctaIdCriteria criteria) {
        return moctaDao.findById(moctaMapper.toMoctaId(criteria))
                .map(moctaMapper::toMoctaDocDto)
                .map(i -> {
                    invmbDao.findById(i.getTa006()).ifPresent(j -> i.setMb029(j.getMb029()));
                    return i;
                });
    }


    public List<MoctaDto> getMoctaList(MoctaCriteria criteria) {
        Specification<Mocta> query = new Specification<Mocta>() {
            @Override
            public Predicate toPredicate(Root<Mocta> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (StringUtils.isNotBlank(criteria.getTa002())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("ta002"), criteria.getTa002() );
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getTa001())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("ta001"), criteria.getTa001() );
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getGtCreate_date())) {
                    Predicate predicate = criteriaBuilder.greaterThan(root.get("create_date"), criteria.getGtCreate_date());
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getGtModi_date())) {
                    Predicate predicate = criteriaBuilder.greaterThan(root.get("modi_date"), criteria.getGtModi_date());
                    predicates.add(predicate);
                }
                return criteriaQuery.where(predicates.toArray(new Predicate[0])).getRestriction();
            }
        };
        List<Mocta> list = moctaDao.findAll(query);
        return moctaMapper.toMoctaDtoList(list);
    }



    public List<MoctbDto> getMoctbList(MoctbCriteria criteria) {
        Specification<Moctb> query = new Specification<Moctb>() {
            @Override
            public Predicate toPredicate(Root<Moctb> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (StringUtils.isNotBlank(criteria.getTb001())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("tb001"), criteria.getTb001() );
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getTb002())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("tb002"), criteria.getTb002() );
                    predicates.add(predicate);
                }
                return criteriaQuery.where(predicates.toArray(new Predicate[0])).getRestriction();
            }
        };
        List<Moctb> list = moctbDao.findAll(query);
        return moctaMapper.toMoctbDto(list);
    }


}
